@@ -21,37 +21,29 @@
 ;;; License along with Dezyne.  If not, see <http://www.gnu.org/licenses/>.
 
 (define-module (dzn code)
-  #:use-module (ice-9 curried-definitions)
-  #:use-module (ice-9 getopt-long)
-  #:use-module (ice-9 match)
-  #:use-module (ice-9 optargs)
-  #:use-module (ice-9 receive)
-
   #:use-module (srfi srfi-1)
   #:use-module (srfi srfi-26)
 
-  #:use-module (dzn misc)
-  #:use-module (dzn command-line)
-  #:use-module (dzn config)
-
-  #:use-module ((oop goops) #:renamer (lambda (x) (if (member x '(<port> <foreign>)) (symbol-append 'goops: x) x)))
-  #:use-module (dzn goops)
-  #:use-module (dzn display)
-  #:use-module (dzn shell-util)
-  #:use-module (dzn config)
+  #:use-module (ice-9 curried-definitions)
+  #:use-module (ice-9 match)
 
+  #:use-module (dzn ast display)
+  #:use-module (dzn ast goops)
+  #:use-module (dzn ast lookup)
+  #:use-module (dzn ast normalize)
   #:use-module (dzn ast)
   #:use-module (dzn code dzn)
+  #:use-module (dzn command-line)
   #:use-module (dzn config)
-  #:use-module (dzn normalize)
-  #:use-module (dzn templates)
+  #:use-module (dzn misc)
   #:use-module (dzn vm goops)
 
   #:declarative? #f
 
   #:export (<port-pair>
-           .other
+            .other
             %calling-context
+            %no-unreachable?
             %shell
             code
             code:add-calling-context
@@ -85,6 +77,7 @@
             code:function-type
             code:functions
             code:global-enum-definer
+            code:in-argument
             code:injected-bindings
             code:injected-instances
             code:injected-instances-system
@@ -115,6 +108,7 @@
             code:requires-in-void-returns
             code:return
             code:return-values
+            code:short-circuit?
             code:trace-q-out
             code:trigger
             code:type-name
@@ -129,6 +123,9 @@
 ;; The calling-context to insert.
 (define %calling-context (make-parameter #f))
 
+;; Should unreachable-code tags be omitted?
+(define %no-unreachable? (make-parameter #f))
+
 ;; The name of the thread-safe shell.
 (define %shell (make-parameter #f))
 
@@ -168,7 +165,7 @@
   (code:interface-include o (ast:source-file o)))
 
 (define-method (code:interface-include (o <foreign>))
-  (code:interface-include o (ast:source-file (parent o <root>))))
+  (code:interface-include o (ast:source-file (ast:parent o <root>))))
 
 (define (code:component-include o)
   (let ((source-file (ast:source-file o)))
@@ -182,11 +179,10 @@
             (ast:instance* o))))
 
 (define-method (code:pump? (o <component>))
-  (and (or (pair? (ast:async-port* o))
-           (pair? (tree-collect (disjoin (is? <blocking>)
-                                         (is? <defer>)
-                                         (is? <blocking-compound>))
-                                o)))
+  (and (pair? (tree-collect (disjoin (is? <blocking>)
+                                     (is? <defer>)
+                                     (is? <blocking-compound>))
+                            o))
        o))
 
 (define-method (code:pump? (o <root>))
@@ -205,7 +201,8 @@
   (code:file-name (.type o)))
 
 (define-method (code:file-name (o <foreign>))
-  (ast:full-name o))
+  (if (ast:imported? o) (next-method)
+      (ast:full-name o)))
 
 (define-method (code:file-name (o <import>))
   (basename (.name o) ".dzn"))
@@ -233,7 +230,7 @@
     (.name (car (ast:provides-port* component)))))
 
 (define-method (code:port-name (o <binding>))
-  (let* ((model (parent o <model>))
+  (let* ((model (ast:parent o <model>))
          (left (.left o))
          (right (.right o))
          (port (and (code:port-bind? o)
@@ -265,7 +262,11 @@
   (ast:full-name o))
 
 (define-method (code:type-name (o <binding>))
-  ((compose code:type-name .type (cut ast:lookup (parent o <model>) <>) injected-instance-name) o))
+  ((compose code:type-name
+            .type
+            (cute ast:lookup (ast:parent o <model>) <>)
+            injected-instance-name)
+   o))
 
 (define-method (code:type-name (o <enum-field>))
   (append (code:type-name (.type o)) (list (.field o))))
@@ -283,6 +284,7 @@
       (($ <extern>) (list (.value type)))
       (($ <bool>) '("bool"))
       (($ <int>) '("int"))
+      (($ <subint>) '("int"))
       (($ <void>) '("void"))
       (_ (ast:full-name type)))))
 
@@ -306,11 +308,26 @@
 (define-method (code:variable-name (o <formal>))
   (make-out-formal o))
 
-(define-method (code:variable-name (o <ast>))
+(define-method (code:variable-name (o <top>))
+  o)
+
+(define-method (code:variable-name (o <assign>))
   ((compose code:variable-name .variable) o))
 
+(define-method (code:variable-name (o <field-test>))
+  ((compose code:variable-name .variable) o))
+
+(define-method (code:variable-name (o <var>))
+  ((compose code:variable-name .variable) o))
+
+(define-method (code:variable-name (o <argument>))
+  o)
+
+(define-method (code:upcase-model-name (o <model>))
+  (map string-upcase (ast:full-name o)))
+
 (define-method (code:upcase-model-name o)
-  (map string-upcase (ast:full-name (parent o <model>))))
+  (code:upcase-model-name (ast:parent o <model>)))
 
 
 ;;;
@@ -325,7 +342,7 @@
         (ast:statement* behavior))))
 
 (define-method (code:ons (o <port>))
-  (let* ((component (parent o <component>))
+  (let* ((component (ast:parent o <component>))
          (behavior (.behavior component))
          (ons (if (not behavior) '()
                   (ast:statement* behavior))))
@@ -336,7 +353,7 @@
 (define-method (code:reply (o <type>))
   o)
 
-(define-method (code:return-type-eq? (a <int>) (b <int>))
+(define-method (code:return-type-eq? (a <subint>) (b <subint>))
   #t)
 (define-method (code:return-type-eq? a b)
   (ast:eq? a b))
@@ -381,16 +398,16 @@
                            (is? <expression>)
                            (is? <location>)))
           (disjoin (is? <blocking>) (is? <blocking-compound>))
-          (parent o <model>))))
+          (ast:parent o <model>))))
 
 (define-method (code:port-release o)
-  (let ((trigger (and=> (parent o <on>)
+  (let ((trigger (and=> (ast:parent o <on>)
                         (compose car ast:trigger*))))
     (and (or (not trigger)
              (ast:requires? trigger)
              (or (not (ast:equal? (.port o) (.port trigger)))
-                 (parent o <blocking>)
-                 (parent o <blocking-compound>)))
+                 (ast:parent o <blocking>)
+                 (ast:parent o <blocking-compound>)))
          (code:blocking? o)
          o)))
 
@@ -401,13 +418,13 @@
 
 (define-method (code:default-true (o <defer>))
   (if (or (and=> (.arguments o) (compose null? .elements))
-          (null? (ast:variable* (parent o <component>)))) o
+          (null? (ast:variable* (ast:parent o <component>)))) o
           '()))
 
 (define-method (code:defer-condition (o <defer>))
   (if (not (or (and=> (.arguments o)(compose null? .elements))
-               (null? (ast:variable* (parent o <component>))))) o
-      '()))
+               (null? (ast:variable* (ast:parent o <component>))))) o
+               '()))
 
 (define-method (code:capture-local (o <defer>))
   (let* ((references (tree-collect (disjoin(is? <assign>)
@@ -416,7 +433,7 @@
                                    (.statement o)))
          (variables (map .variable references))
          (local? (compose (cute ast:eq? <> o)
-                          (cute parent <> <defer>))))
+                          (cute ast:parent <> <defer>))))
     (filter (negate (disjoin ast:member? local?))
             variables)))
 
@@ -449,24 +466,24 @@
   (or (as (ast:type (.expression o)) <void>)
       (.expression o)))
 
-(define-method (code:arguments (o <call>))
+(define-method (code:arguments (o <ast>) (signature <signature>))
   (map code:variable->argument
        (code:add-calling-context-argument (ast:argument* o))
-       (ast:formal* (code:add-calling-context-formal ((compose .formals .signature .function) o)))))
+       (ast:formal* (code:add-calling-context-formal
+                     (.formals signature)))))
+
+(define-method (code:arguments (o <call>))
+  (code:arguments o (.signature (.function o))))
 
 (define-method (code:arguments (o <action>))
-  (if (and (ast:async? o)
-           (equal? (.event.name o) "clr"))
-      (map code:variable->argument
-           (ast:argument* o)
-           (ast:formal* ((compose .formals .signature .event) o)))
-      (map code:variable->argument
-           (code:add-calling-context-argument (ast:argument* o))
-           (ast:formal* (code:add-calling-context-formal ((compose .formals .signature .event) o))))))
+  (code:arguments o (.signature (.event o))))
 
 (define-method (code:arguments (o <trigger>))
   (code:formals o))
 
+(define-method (code:in-argument (o <trigger>))
+  (filter ast:in? (code:formals o)))
+
 (define-method (code:out-argument (o <trigger>))
   (filter (disjoin ast:out? ast:inout?) (code:formals o)))
 
@@ -497,26 +514,33 @@
                              (map .name (ast:formal* formals))
                              (ast:formal* event)))))))
 
-(define-method (code:variable->argument (o <variable>) (f <formal>))
-  (if (or (code:class-member? o)
-          (eq? (.direction f) 'in)) o
-          (clone (make <argument> #:name (.name o) #:type.name (.type.name f) #:direction (.direction f))
-                 #:parent (.parent o))))
+(define-method (code:variable->argument (o <expression>) (v <variable>) (f <formal>))
+  (if (or (code:class-member? v) (eq? (.direction f) 'in)) v
+      (let ((argument (make <argument>
+                        #:name (.name v)
+                        #:type.name (.type.name f)
+                        #:direction (.direction f)
+                        #:expression o)))
+        (clone argument #:parent (.parent o)))))
+
+(define-method (code:variable->argument (o <expression>) (v <formal>) (f <formal>))
+  (if (eq? (.direction f) 'in) v
+      (let ((argument (make <argument>
+                        #:name (.name v)
+                        #:type.name (.type.name v)
+                        #:direction (.direction v)
+                        #:expression o)))
+        (clone argument #:parent (.parent o)))))
 
 (define-method (code:variable->argument (o <var>) (f <formal>))
-  (code:variable->argument (.variable o) f))
+  (code:variable->argument o (.variable o) f))
 
 (define-method (code:variable->argument (o <formal>) (f <formal>))
-  (if (eq? (.direction f) 'in) o
-      (clone (make <argument> #:name (.name o) #:type.name (.type.name o) #:direction (.direction o))
-             #:parent (.parent o))))
+  (code:variable->argument o o f))
 
 (define-method (code:variable->argument o f)
   o)
 
-(define-method (code:variable-name (o <argument>))
-  o)
-
 (define-method (code:data* (o <root>))
   (filter (negate ast:imported?) (ast:data* o)))
 
@@ -560,7 +584,7 @@
           (ast:type* o)))
 
 (define-method (code:global-enum-definer (o <model>))
-  (filter (is? <enum>) (ast:type* (parent o <root>))))
+  (filter (is? <enum>) (ast:type* (ast:parent o <root>))))
 
 (define-method (code:enum-literal (o <enum-literal>))
   (append (code:type-name (.type o)) (list (.field o))))
@@ -568,8 +592,8 @@
 (define-method (code:enum-scope (o <enum-literal>))
   (let* ((enum (.type o))
          (scope (ast:full-scope enum))
-         (model-scope (and=> (parent o <model>) ast:full-name)))
-    (cond ((or (null? scope) (null? model-scope)) (parent enum <root>))
+         (model-scope (and=> (ast:parent o <model>) ast:full-name)))
+    (cond ((or (null? scope) (null? model-scope)) (ast:parent enum <root>))
           ((equal? scope model-scope) (make <model-scope> #:scope model-scope))
           (else enum))))
 
@@ -595,7 +619,7 @@
   '())
 
 (define-method (code:instance-name (o <binding>))
-  (let* ((model (parent o <model>))
+  (let* ((model (ast:parent o <model>))
          (left (.left o))
          (right (.right o))
          (bind (and (code:port-bind? o)
@@ -660,7 +684,7 @@
   (filter code:port-bind? (filter (negate injected-binding?) (ast:binding* o))))
 
 (define-method (code:bind-provides-required (o <binding>))
-  (let* ((model (parent o <model>))
+  (let* ((model (ast:parent o <model>))
          (left (.left o))
          (left-port (.port left))
          (right (.right o))
@@ -761,23 +785,34 @@
                        (lambda (a b)
                          (ast:eq? (.port a) (.port b))))))
 
+(define (code:short-circuit? o)
+  (match o
+    ((or ($ <foreign>) ($ <system>))
+     o)
+    ((and (or ($ <interface>) ($ <component>)) (? ast:imported?))
+     o)
+    (_
+     #f)))
+
 (define (code:normalize ast)
-  (let ((root ((compose
-                add-reply-port
-                normalize:event+illegals
-                remove-otherwise
-                (binding-into-blocking)
-                code:add-calling-context)
-               ast)))
-    (when (> (dzn:debugity) 1)
-      (ast:pretty-print root (current-error-port)))
-    root))
+  (parameterize ((%normalize:short-circuit? code:short-circuit?))
+    (let ((root ((compose
+                  add-reply-port
+                  normalize:event+illegals
+                  remove-otherwise
+                  (binding-into-blocking)
+                  code:add-calling-context)
+                 ast)))
+      (when (> (dzn:debugity) 1)
+        (ast:pretty-print root (current-error-port)))
+      root)))
 
 (define (code:normalize+determinism ast)
-  ((compose
-    add-determinism-temporaries
-    code:normalize)
-   ast))
+  (parameterize ((%normalize:short-circuit? code:short-circuit?))
+    ((compose
+      add-determinism-temporaries
+      code:normalize)
+     ast)))
 
 
 ;;;
